#include <progresswin.h>
#include <glib/gi18n.h>

/*
progress_window::
new();
destroy();
*/


static gboolean progress_window_ondelete (GtkWidget *widget,
                                            GdkEvent *event,
                                            gpointer user_data)
{
	struct ProgressWindow* window = (struct ProgressWindow*) user_data;

	printf("progress window deleted\n");
	if (window->callback) {
		window->callback(window->callback_context);
		return FALSE;
	} else
		return TRUE; //refuse to close
}

static void progress_window_oncancel(GtkWidget* button, void * user_data)
{
	struct ProgressWindow* window = (struct ProgressWindow*) user_data;

	printf("progress window cancelled\n");
	if (window->callback)
		window->callback(window->callback_context);
}

struct ProgressWindow* progress_window_new(char* filename, void(*abortfunc)(void*), void* userdata)
{
	struct ProgressWindow* window;
	GtkWidget* frame;
	GtkWidget* widget, * prgsbar;

	window = (struct ProgressWindow*)malloc(sizeof(struct ProgressWindow));
	window->callback = abortfunc;
	window->callback_context = userdata;
	window->filename = strdup(filename);
	
	//window
	window->handle = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(window->handle, _("Beaming Progress"));
	gtk_container_set_border_width (GTK_CONTAINER (window->handle), 10);
	g_signal_connect(G_OBJECT(window->handle),
                   "delete-event",
                   G_CALLBACK(progress_window_ondelete),
                   window);

	frame = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window->handle), frame);

	//label
	window->label = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(frame), window->label, FALSE, FALSE, 0);
	
	//progress bar
	window->progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_text(window->progressbar, filename);
	gtk_box_pack_start(GTK_BOX(frame), window->progressbar, TRUE, FALSE, 10);

	//cancel button
	widget = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	g_signal_connect(G_OBJECT(widget),
                   "clicked",
                   G_CALLBACK(progress_window_oncancel),
                   window);

	gtk_box_pack_start(GTK_BOX(frame), widget, TRUE, FALSE, 10);

	gtk_widget_show_all(window->handle);
	
	return window;
}

void progress_window_destroy(struct ProgressWindow* window)
{
	gtk_widget_destroy(window->handle);
	free(window);
}

void progress_window_update(struct ProgressWindow* window, int current, int total)
{
	char labeltext[100];
	if(total>0)
		gtk_progress_bar_set_fraction(window->progressbar, (double)current/total);
	else
		gtk_progress_bar_pulse(window->progressbar);
	
	/*if(total>0)
		sprintf(labeltext, "<b>Progress:</b> %d of %d", current, total);
	else
		sprintf(labeltext, "<b>Progress:</b> %d", current);*/
	sprintf(labeltext, _("<b>Transfering</b> %s"), window->filename);
	gtk_label_set_markup (window->label, labeltext);

	if(total>0) {
		sprintf(labeltext, _("%d of %d (%d%%)"), current, total, current*100/total);
		gtk_progress_bar_set_text(window->progressbar, labeltext);
	} else {
		sprintf(labeltext, _("%d bytes"), current);
		gtk_progress_bar_set_text(window->progressbar, window->filename);
	}
}
